function* mapIterable(iterable, callback) {
	for (const item of iterable) yield callback(item)
}
function* filterIterable(iterable, callback) {
	for (const item of iterable) callback(item) && (yield item)
}
function* toGenerator(iterable) {
	for (const item of iterable) yield item
}
function* iterateInParallel(firstIterable, secondIterable) {
	const secondIterator = secondIterable[Symbol.iterator]()
	for (const first of firstIterable) {
		const secondResult = secondIterator.next()
		if (secondResult.done) throw Error("Second iterable finished before first")
		yield [first, secondResult.value]
	}
	if (!secondIterator.next().done) throw Error("First iterable finished before second")
}
async function asyncIterableToArray(asyncIterable) {
	const array = []
	for await (const item of asyncIterable) array.push(item)
	return array
}
async function* mergeAsyncIterators(asyncIterators) {
	const promises = asyncIterators.map(asyncGenerator => {
		const promise = asyncGenerator.next().then(iteratorResult => ({ iteratorResult, asyncGenerator, promise }))
		return promise
	})
	for (; asyncIterators.length; ) {
		const { iteratorResult, asyncGenerator, promise } = await Promise.race(promises)
		if (iteratorResult.done) {
			promises.splice(promises.indexOf(promise), 1)
			continue
		}
		yield iteratorResult.value
		const newPromise = asyncGenerator
			.next()
			.then(iteratorResult => ({ iteratorResult, asyncGenerator, promise: newPromise }))
		promises[promises.indexOf(promise)] = newPromise
	}
}
export { asyncIterableToArray, filterIterable, iterateInParallel, mapIterable, mergeAsyncIterators, toGenerator }
