import { wait } from "./wait.js"
async function retry(
	callback,
	{
		attempts = 3,
		retryDelay = 1e3,
		onError = error => console.error("Caught", error, `retrying ${attempts} more time(s) in ${retryDelay}ms`),
		exponentialBackoff = !0
	} = {}
) {
	if (attempts < 1) throw Error("Retry attempts must be higher than 0")
	for (; --attempts; ) {
		try {
			return await callback()
		} catch (error) {
			onError(error)
		}
		await wait(retryDelay)
		exponentialBackoff && (retryDelay *= 2)
	}
	return callback()
}
export { retry }
